{
********************************************************************************
*  Name    : Internal Oscillator Module                                        *
*  Author  : AndyO (some additions by Graham from digital-diy.com)             *
*  Notice  : Copyright (c) 2008 AndyO                                          *
*          : All Rights Reserved                                               *
*  Date    : 26th Jun 2010                                                     *
*  Version : 1.3                                                               *
*  Notes   : Sets Config fuses and OSCCON register for internal oscillator     *
*          : operation with IO on RA6 & RA7                                    *
*          :                                                                   *
*          : Module include should be before any other modules in main program *
*          :                                                                   *
*          : Supported devices: 18F1220, 18F1320, 18F2420, 18F2520, 18F4420,   *
*          : 18F4520, 18F2455, 18F2550, 18F4455, 18F4550, 18F2620, 18F4685     *
*          : 18F25K20                                                          *

*          : 18F25K20: Vr 2.1.0.2 of SF produces errors below 8Mhz             *
********************************************************************************
}

Module InternalOscillator 

'Set clock source to internal oscillator, I/O on RA6 & RA7
#if _device in (18F1220, 18F1320)
    
    Config OSC = INTIO2

#elseif _device in (18F2420, 18F2520, 18F4420, 18F4520)
    
    Config OSC = INTIO7
    
#elseif _device in (18F2455, 18F2550, 18F4455, 18F4550)

    Config FOSC = INTOSCIO_EC
    
#elseif _device in (18F2620, 18F4620) Then

     Config OSC = INTIO67
     
#elseif _device in (18F4685) Then

    Config OSC = IRCIO67
    
#elseif _device in (18F25K20) Then

     Config FOSC = INTIO67    
     
#else

    #error "Can not configure OSC fuse for chosen device"
    
#endif


'Set-up internal oscillator
#if _device in (18F25K20)
    #if _clock = 16
        
        OSCCON = $70
    
    #elseif _clock = 8
    
        OSCCON = $60
        
    #elseif _clock = 4
    
        OSCCON = $50
        
    #elseif _clock = 2
    
        OSCCON = $40
        
    #elseif _clock = 1
    
        OSCCON = $30
        
    #elseif _clock = 0.5
    
        OSCCON = $20
        
    #elseif _clock = 0.25
    
        OSCCON = $10
    
    #elseif _clock = 0.031
    
        OSCCON = $00
    
    #elseif _clock = 32 And _device in (18F25K20)
    
        OSCCON = $60                                'Set internal oscillator to 8MHz
        OSCTUNE.6 = 1                               'Enables PLL
        
    #elseif _clock = 64 And _device in (18F25K20)
    
        OSCCON = $70                                'Set internal oscillator to 16MHz
        OSCTUNE.6 = 1                               'Enables PLL
        
    #else
    
        #error "Can not configure OSCCON for chosen clock speed" 
        
    #endif
#else
    #if _clock = 8
        
        OSCCON = $7F
    
    #elseif _clock = 4
    
        OSCCON = $6F
        
    #elseif _clock = 2
    
        OSCCON = $5F
        
    #elseif _clock = 1
    
        OSCCON = $4F
        
    #elseif _clock = 0.5
    
        OSCCON = $3F
        
    #elseif _clock = 0.25
    
        OSCCON = $2F
        
    #elseif _clock = 0.125
    
        OSCCON = $1F
    
    #elseif _clock = 0.031
    
        OSCCON = $0F
    
    #elseif _clock = 16 And _device in (18F2420, 18F2520, 18F2620, 18F4420, 18F4520, 18F4620, 18F4685)
    
        OSCCON = $6C                                'Set internal oscillator to 4MHz
        OSCTUNE.6 = 1                               'Enables PLL
        
    #elseif _clock = 32 And _device in (18F2420, 18F2520, 18F2620, 18F4420, 18F4520, 18F4620, 18F4685)
    
        OSCCON = $7C                                'Set internal oscillator to 8MHz
        OSCTUNE.6 = 1                               'Enables PLL
        
    #else
    
        #error "Can not configure OSCCON for chosen clock speed" 
        
    #endif
#endif  
